% DEMODCT6.M
% Perform DCT analysis of Lenna using 8x8 DCTs with different quantiser
% strategies (qstep=15,30 and Qjpeg, Qjpeg*2), and calculate entropies etc.

clear all
global printon
printon = input('Write print files (0=no, 1=yes):');

% JPEG quantisation table for luminance.
qlum = jpegqtab;

qstep = qlum;

% 1-D DCT matrix.
T = dctmat(8);

load lenna
X = X - 120;

disp('Doing DCT'); X=colxfm(colxfm(X,T)',T)'; 
X = regroup(X,8);

% save lendct X

% Quantise using qstep=15.
qstep = 15;
[Y,ent8a,rmsea] = dctquant(X,qstep,8);
ent8m = mean(mean(ent8a));
rmse = rms(rmsea(:));

% Perform inverse transform.
Y = regroup(Y,32);
disp('Doing Inv-DCT'); Y = colxfm(colxfm(Y',T')',T');

figure(1)
draw(Y,[],[260 260])
set(gcf,'numbertitle','off','name','Fig 1: Qstep = 15')
drawnow
if printon==1,
  print -deps \1ngk\tex\i8coding\i8dct61.eps
end

% Quantise using qstep=30.
qstep = 30;
[Y,ent8b,rmseb] = dctquant(X,qstep,8);
ent8m(2) = mean(mean(ent8b));
rmse(2) = rms(rmseb(:));

% Perform inverse transform.
Y = regroup(Y,32);
disp('Doing Inv-DCT'); Y = colxfm(colxfm(Y',T')',T');

figure(2)
draw(Y,[],[520 260])
set(gcf,'numbertitle','off','name','Fig 2: Qstep = 30')
drawnow
if printon==1,
  print -deps \1ngk\tex\i8coding\i8dct62.eps
end

% Quantise using qstep=qlum.
qstep = qlum;
[Y,ent8c,rmsec] = dctquant(X,qstep,8);
ent8m(3) = mean(mean(ent8c));
rmse(3) = rms(rmsec(:));

% Perform inverse transform.
Y = regroup(Y,32);
disp('Doing Inv-DCT'); Y = colxfm(colxfm(Y',T')',T');

figure(3)
draw(Y,[],[260 0])
set(gcf,'numbertitle','off','name','Fig 3: Qstep = JPEG lum.')
drawnow
if printon==1,
  print -deps \1ngk\tex\i8coding\i8dct63.eps
end

% Quantise using qstep=qlum*2.
qstep = qlum*2;
[Y,ent8d,rmsed] = dctquant(X,qstep,8);
ent8m(4) = mean(mean(ent8d))
rmse(4) = rms(rmsed(:))

% Perform inverse transform.
Y = regroup(Y,32);
disp('Doing Inv-DCT'); Y = colxfm(colxfm(Y',T')',T');

figure(4)
draw(Y,[],[520 0])
set(gcf,'numbertitle','off','name','Fig 4: Qstep = 2*JPEG lum.')
drawnow
if printon==1,
  print -deps \1ngk\tex\i8coding\i8dct64.eps
end

setfig(5);
subplot(2,2,1);
plot(ent8a');
axis('normal');
axis([0 8 0 8]);
grid on
title('DCT entropy for Qstep=15');
ylabel('Entropy (bit/pel)');
text(4,7,sprintf('Mean=%.4f',ent8m(1)),'vert','mid','horiz','cent');
text(4,5,sprintf('RMSE=%.4f',rmse(1)),'vert','mid','horiz','cent');

subplot(2,2,2);
plot(ent8b');
axis('normal');
axis([0 8 0 8]);
grid on
title('DCT entropy for Qstep=30');
ylabel('Entropy (bit/pel)');
text(4,7,sprintf('Mean=%.4f',ent8m(2)),'vert','mid','horiz','cent');
text(4,5,sprintf('RMSE=%.4f',rmse(2)),'vert','mid','horiz','cent');

subplot(2,2,3);
plot(ent8c');
axis('normal');
axis([0 8 0 8]);
grid on
title('DCT entropy for Qstep=JPEG lum.');
ylabel('Entropy (bit/pel)');
text(4,7,sprintf('Mean=%.4f',ent8m(3)),'vert','mid','horiz','cent');
text(4,5,sprintf('RMSE=%.4f',rmse(3)),'vert','mid','horiz','cent');

subplot(2,2,4);
plot(ent8d');
axis('normal');
axis([0 8 0 8]);
grid on
title('DCT entropy for Qstep=2*JPEG lum.');
ylabel('Entropy (bit/pel)');
text(4,7,sprintf('Mean=%.4f',ent8m(4)),'vert','mid','horiz','cent');
text(4,5,sprintf('RMSE=%.4f',rmse(4)),'vert','mid','horiz','cent');

drawnow

if printon==1,
  print -deps \1ngk\tex\i8coding\i8dct65.eps
end

