% DEMODCT2.M
% Perform DCT analysis of Lenna and calculate entropies etc.

global printon
% printon = input('Write print files (0=no, 1=yes, 2=GIF):');
printon = 0;

% 1-D DCT matrix.
T = dctmat(8);

% Generate a matrix of 2-D dct basis functions.
dct2d = zeros(8*9+1,8*9+1) - 1/4;

t = [1:8] - 8;
for r=1:8,
  for c=1:8,
    dct2d(t+r*9,t+c*9) = T(r,:)' * T(c,:);
  end
end
dct2d = dct2d*512;


figure(1)
draw(dct2d)
drawnow
if printon==2,
  drawfile(dct2d,'\1ngk\tex\i8coding\i8dct2.gif');
elseif printon==1,
  print -deps \1ngk\tex\i8coding\i8dct2.eps
else
  pause
end

clear dct2d
load lenna
X = X - 120;

disp('Doing DCT'); X=colxfm(colxfm(X,T)',T)'; 

figure(2)
draw(X)
if printon==2,
  drawfile(X,'\1ngk\tex\i8coding\i8dct3.gif');
elseif printon==1,
  print -deps \1ngk\tex\i8coding\i8dct3.eps
else
  pause
end

X = regroup(X,8);
figure(3)
draw(X)
if printon==2,
  drawfile(X,'\1ngk\tex\i8coding\i8dct4.gif');
elseif printon==1,
  print -deps \1ngk\tex\i8coding\i8dct4.eps
else
  pause
end

figure(4)
t=1:128;draw(X(t,t))
pause

figure(5)
t=1:64;draw(X(t,t))
if printon==2,
  drawfile(X(t,t),'\1ngk\tex\i8coding\i8dct5.gif');
elseif printon==1,
  print -deps \1ngk\tex\i8coding\i8dct5.eps
else
  pause
end

% Calculate and plot entropies of top left 4 subimages.
figure(6);
[e3, ehuff3] = ent4plot(X(t,t),15, ...
 'DCT top left 4 bands, energies and entropies for Qstep=15', ...
 [-60 60 0 0.6], 0);
if printon==1,
  print -deps \1ngk\tex\i8coding\i8dct6.eps
end

% Plot the 64 entropies of X using mesh and plot.
ent8 = dctent(X,15,8);
save ent8 ent8

mat2scr(ent8,'%8.4f','8 x 8 DCT entropy matrix');
mat2scr(mean(ent8(:)),'Mean entropy = %.4f bits');

setfig(7);
subplot(1,2,1);
mesh(ent8');
view(110,20);
axis('square');
axis([0 10 0 10 0 8]);
grid on
title('Mesh plot of DCT entropy','FontSize',16);

subplot(1,2,2);
plot(ent8');
axis('square');
axis([0 8 0 8]);
grid on
title('Row plots of DCT entropy','FontSize',16);
xlabel('column','FontSize',16);
ylabel('Entropy (bit/pel)','FontSize',16);
text(1,ent8(1,1),'row 1','horiz','center','vert','bot');
text(1,ent8(2,1),'2 ','horiz','right','vert','mid');
text(1,ent8(3,1),'3 ','horiz','right','vert','mid');
text(1,ent8(8,1),'8 ','horiz','right','vert','mid');
if printon==1,
  print -deps \1ngk\tex\i8coding\i8dct7.eps
end

