% DEMODCT3.M
% Perform DCT analysis of Lenna using 4x4 DCTs and
% calculate entropies etc.

close all
global printon
% printon = input('Write print files (0=no, 1=yes, 2=GIF):');
printon = 0;

% 1-D DCT matrix.
T = dctmat(4);

load lenna
X = X - 120;

disp('Doing DCT'); X=colxfm(colxfm(X,T)',T)'; 

X = regroup(X,4);

% Fig 3.2c: DCT reordered into subimages grouped by coefficient type.
figure(1)
draw(X)
drawnow
if printon==2,
  drawfile(X,'\1ngk\tex\i8coding\i8dct31.gif');
elseif printon==1,
  print -deps \1ngk\tex\i8coding\i8dct31.eps
end

% Plot the 16 entropies of X using mesh and plot.
ent4 = dctent(X,15,4);
save ent4 ent4

mat2scr(ent4,'%8.4f','4 x 4 DCT entropy matrix');
mat2scr(mean(ent4(:)),'Mean entropy = %.4f bits');

if printon==0,
  pause
end

setfig(2);
subplot(1,2,1);
mesh(ent4');
view(110,20);
axis('square');
axis([0 5 0 5 0 8]);
grid on
title('Mesh plot of DCT entropy','FontSize',16);

subplot(1,2,2);
plot(ent4');
axis('square');
axis([0 4 0 8]);
grid on
title('Row plots of DCT entropy','FontSize',16);
xlabel('column','FontSize',16);
ylabel('Entropy (bit/pel)','FontSize',16);
text(1,ent4(1,1),'row 1','horiz','center','vert','bot');
text(1,ent4(2,1),'2 ','horiz','right','vert','mid');
text(1,ent4(3,1),'3 ','horiz','right','vert','mid');
text(1,ent4(4,1),'4 ','horiz','right','vert','mid');
if printon==1,
  print -deps \1ngk\tex\i8coding\i8dct32.eps
end

