% DEMWAV1A.M
% Plot impulse and frequency responses of the following wavelets:
%   Haar wavelet,
%   Le Gall analysis and reconstruction wavelets,
%   Daubechies 4-tap wavelets,
%   Tay/Kingsbury near-balanced 5,7-tap wavelets,
%   Tay/Kingsbury near-balanced 13,19 tap wavelets.
%   Antonini 7,9-tap wavelets (Trans IP, April 92, p 205).

clear all
close all
titlfont = 14;

alpha = sqrt(3) - 2;
% \frac{1}{2\sqrt{1 + \alpha^2}}(1 + z^{-1})^2\ (1 + \alpha z^{-1})
h0daub = conv([1 2 1]',[1 alpha]') / (2 * sqrt(1 + alpha^2))
h1daub = h0daub(4:-1:1) .* [-1 1 -1 1]'

global printon
printon = input('Write print files (0=no, 1=yes):');

% Start with the Haar wavelet.

h0 = [1;1]/sqrt(2);
h1 = [-1;1]/sqrt(2);
h = wav4(h0,h1);

setfig(1)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Haar 2-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 1: ' 'Haar 2-tap wavelets'])
xlabel('sample number');
th = -6; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep+0.25,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
tv = 0.05;
text(3.8,H(512,1)+tv,'H1','vert','bot','horiz','right');
text(1.7,H(210,2)+tv,'H01','vert','bot','horiz','center');
text(0.85,H(110,3)+tv,'H001','vert','bot','horiz','left');
text(0.425,H(55,4)+tv,'H0001','vert','bot','horiz','left');
text(0.2,H(10,5)+tv,'H0000','vert','bot','horiz','left');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav1.eps
end


% Now do the Le Gall wavelet.
h0 = [1 2 1]'/sqrt(6);
h1 = [-1 -2 6 -2 -1]'/sqrt(46);

h = wav4(h0,h1);

setfig(2)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('LeGall 3,5-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 2: ' 'LeGall 3,5-tap wavelets'])
xlabel('sample number');
th = -16; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep+0.15,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav2.eps
end

% This is the other Le Gall wavelet - less smooth.
h1 = [-1 2 -1]'/sqrt(6);
h0 = [-1 2 6 2 -1]'/sqrt(46);

h = wav4(h0,h1);

setfig(3)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('LeGall 5,3-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 3: ' 'LeGall 5,3-tap wavelets'])
xlabel('sample number');
th = -20; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav3.eps
end


% Now do the Antonini wavelet.
% Antonini 7,9-tap wavelets (FBI standard).
% This polynomial makes even terms of P(z) zero when other factors 
% are (1+z)^8.
r2 = roots([5 -40 131 -208 131 -40 5]);
% 4 roots at z=-1 and the 4 complex roots from r2.
h0ant = real(poly([-1; -1; -1; -1; r2(2:5)])');
h0ant = sqrt(2)*h0ant/sum(h0ant);  % sqrt(2) DC gain.
% 4 roots at z=-1 and the 2 real roots from r2.
g0ant = real(poly([-1; -1; -1; -1; r2([1 6])])');
g0ant = sqrt(2)*g0ant/sum(g0ant);  % sqrt(2) DC gain.
% Generate hi-band filters as quadrature mirrors of lo-band filters.
h1ant = g0ant .* cumprod(-ones(size(g0ant)));
g1ant = -h0ant .* cumprod(-ones(size(h0ant)));

h = wav4(g0ant,g1ant);

setfig(4)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Antonini 7,9-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 4: ' 'Antonini 7,9-tap wavelets'])
xlabel('sample number');
th = -16; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep+0.15,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav79.eps
end

% This is the other Antonini wavelet - less smooth.
h = wav4(h0ant,h1ant);

setfig(5)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Antonini 9,7-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 5: ' 'Antonini 9,7-tap wavelets'])
xlabel('sample number');
th = -20; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav97.eps
end

% Antonini 6,10-tap wavelets.
% This polynomial makes even terms of P(z) zero when other factors 
% are (1+z)^8.
r2 = roots([5 -40 131 -208 131 -40 5]);
% 5 roots at z=-1 and the 4 complex roots from r2.
h0ant = real(poly([-1; -1; -1; -1; -1; r2(2:5)])');
h0ant = sqrt(2)*h0ant/sum(h0ant);  % sqrt(2) DC gain.
% 3 roots at z=-1 and the 2 real roots from r2.
g0ant = real(poly([-1; -1; -1; r2([1 6])])');
g0ant = sqrt(2)*g0ant/sum(g0ant);  % sqrt(2) DC gain.
% Generate hi-band filters as quadrature mirrors of lo-band filters.
h1ant = g0ant .* cumprod(-ones(size(g0ant)));
g1ant = -h0ant .* cumprod(-ones(size(h0ant)));

h = wav4(g0ant,g1ant);

setfig(6)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Antonini 6,10-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 6: ' 'Antonini 6,10-tap wavelets'])
xlabel('sample number');
th = -16; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep+0.15,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav2b.eps
end

% This is the other Antonini wavelet - less smooth.
h = wav4(h0ant,h1ant);

setfig(7)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Antonini 10,6-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 7: ' 'Antonini 10,6-tap wavelets'])
xlabel('sample number');
th = -20; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav3b.eps
end

% return

% Now do the Daubechies 4-tap wavelets.
h1 = h1daub;
h0 = h0daub;

h = wav4(h0,h1);

setfig(8)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Daubechies 4-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 8: ' 'Daubechies 4-tap wavelets'])
xlabel('sample number');
th = -16; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep+0.2,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav4.eps
end

% Now do the Daubechies 8-tap wavelets (symlets).
[h0,h1,g0,g1] = wavegen('Daub88');
h0 = h0 * sqrt(2);
h1 = h1 * sqrt(2);

h = wav4(h0,h1);

setfig(9)
% figure(9);
subplot(1,2,1)
sep=1.2; trunc = 12;
plotsep(h(1+trunc:end-trunc,:),sep);
title('Daubechies 8-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 9: ' 'Daubechies 8-tap wavelets'])
xlabel('sample number');
th = -26; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep+0.2,'h0000','vert','bot','horiz','center');
% axis([(size(h,1)+1)/2*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav4b.eps
end

% Now do the 5,7-tap and 13,19-tap Transformation of variables filters.
c = -3.5;
a = 2*c + 2/(2 + c);
b = -(2 + c);
Ht = conv([1 1]',-[1 b a]');
Ft = conv([1 1]',-[1 c]');


% Define the transformation function Z = M(z) (even coefs must be 0).
% for the 5,7-tap filters.
ma=0; mb=0;
if ma > 0,
  if mb > 0,
    M = [mb 0 -ma 0 1+ma-mb 0 1+ma-mb 0 -ma 0 mb]'/2;
  else M = [-ma 0 1+ma 0 1+ma 0 -ma]'/2;
  end
else M = [1 0 1]'/2;
end
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ft,M);
h1 = xfm(Ht,-M);

% Normalise to unit energy.
h0 = h0/sqrt(sum(h0 .* h0));
h1 = h1/sqrt(sum(h1 .* h1));

h = wav4(h0,h1);

setfig(10)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Near-balanced 5,7-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 5: ' 'Near-balanced 5,7-tap wavelets'])
xlabel('sample number');
th = -16; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep,'h0000','vert','bot','horiz','center');
axis([25*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav5.eps
end

% Now swap the filters to demonstrate syymetry between analysis
% and reconstruction filters.
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ht,M);
h1 = xfm(Ft,-M);

% Normalise to unit energy.
h0 = h0/sqrt(sum(h0 .* h0));
h1 = h1/sqrt(sum(h1 .* h1));

h = wav4(h0,h1);

setfig(11)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Near-balanced 7,5-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 6: ' 'Near-balanced 7,5-tap wavelets'])
xlabel('sample number');
th = -16; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep,'h0000','vert','bot','horiz','center');
axis([25*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav6.eps
end

% Define the transformation function Z = M(z) (even coefs must be 0).
% for the 13,19-tap filters.
ma=3/16; mb=0;
if ma > 0,
  if mb > 0,
    M = [mb 0 -ma 0 1+ma-mb 0 1+ma-mb 0 -ma 0 mb]'/2;
  else M = [-ma 0 1+ma 0 1+ma 0 -ma]'/2;
  end
else M = [1 0 1]'/2;
end
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ft,M);
h1 = xfm(Ht,-M);

% Normalise to unit energy.
h0 = h0/sqrt(sum(h0 .* h0));
h1 = h1/sqrt(sum(h1 .* h1));

h = wav4(h0,h1);

setfig(12)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Near-balanced 13,19-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 7: ' 'Near-balanced 13,19-tap wavelets'])
xlabel('sample number');
th = -26; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep,'h0000','vert','bot','horiz','center');
axis([40*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav7.eps
end

% Now swap the filters to demonstrate syymetry between analysis
% and reconstruction filters.
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ht,M);
h1 = xfm(Ft,-M);

% Normalise to unit energy.
h0 = h0/sqrt(sum(h0 .* h0));
h1 = h1/sqrt(sum(h1 .* h1));

h = wav4(h0,h1);

setfig(13)
subplot(1,2,1)
sep=1.2;
plotsep(h,sep);
title('Near-balanced 19,13-tap wavelets','Fontsize',titlfont);
set(gcf,'numbertitle','off','name',['Fig 8: ' 'Near-balanced 19,13-tap wavelets'])
xlabel('sample number');
th = -26; tv = 0.1;
text(th,tv,'h1','vert','bot','horiz','center');
text(th,tv-sep,'h01','vert','bot','horiz','center');
text(th,tv-2*sep,'h001','vert','bot','horiz','center');
text(th,tv-3*sep,'h0001','vert','bot','horiz','center');
text(th,tv-4*sep,'h0000','vert','bot','horiz','center');
axis([40*[-1 1]  sep*[-4.5 1]]);

subplot(1,2,2)
H=abs(fft(h,1024));
f=[0:512]/128;
plot(f,H(1:513,:));
title('Frequency responses','Fontsize',titlfont);
xlabel('frequency * (8 / fs)');
axis([0 4 -0.5 max(H(:))+0.5]);

if printon==1,
  print -depsc i8wav8.eps
end
