% DEMWAV4.M
% Calculate the following wavelets:
%   Haar wavelet,
%   Le Gall analysis and reconstruction wavelets,
%   Dabechies 4-tap wavelets,
%   Tay/Kingsbury near-balanced 5,7-tap wavelets,
%   Tay/Kingsbury near-balanced 13,19-tap wavelets (ma = 2/16).

clear all
close all


% Start with the Haar wavelet.

h0haar = [1;1]/sqrt(2);
h1haar = [1;-1]/sqrt(2);

% Now do the Le Gall wavelet.
h0legall = [1 2 1]'/sqrt(8);
h1legall = [-1 -2 6 -2 -1]'/sqrt(32);

% This is the other Le Gall wavelet - less smooth.
g1legall = [-1 2 -1]'/sqrt(8);
g0legall = [-1 2 6 2 -1]'/sqrt(32);

% Define the Daubechies 4-tap wavelet.
alpha = sqrt(3) - 2;
h0daub = conv([1 2 1]',[1 alpha]') / (2 * sqrt(1 + alpha^2));
h1daub = h0daub(4:-1:1) .* [-1 1 -1 1]';

% Now do the 5,7-tap and 13,19-tap Transformation of variables filters.
c = -3.5;
a = 2*c + 2/(2 + c);
b = -(2 + c);
Ht = conv([1 1]',-[1 b a]');
Ft = conv([1 1]',-[1 c]');

% Define the transformation function Z = M(z) (even coefs must be 0).
% for the 5,7-tap filters.
M = [1 0 1]'/2;

% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ft,M);
h1 = xfm(Ht,-M);

% Normalise to unit energy.
h0_57 = h0/sqrt(sum(h0 .* h0));
h1_57 = h1/sqrt(sum(h1 .* h1));

% Now swap the filters to demonstrate symmetry between analysis
% and reconstruction filters.
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ht,M);
h1 = xfm(Ft,-M);

% Normalise to unit energy.
h0_75 = h0/sqrt(sum(h0 .* h0));
h1_75 = h1/sqrt(sum(h1 .* h1));

% Define the transformation function Z = M(z) (even coefs must be 0).
% for the 13,19-tap filters.
ma=2/16;
M = [-ma 0 1+ma 0 1+ma 0 -ma]'/2;

% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ft,M);
h1 = xfm(Ht,-M);

% Normalise to unit energy.
h0_13 = h0/sqrt(sum(h0 .* h0));
h1_19 = h1/sqrt(sum(h1 .* h1));

% Now swap the filters to demonstrate symmetry between analysis
% and reconstruction filters.
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ht,M);
h1 = xfm(Ft,-M);

% Normalise to unit energy.
h0_19 = h0/sqrt(sum(h0 .* h0));
h1_13 = h1/sqrt(sum(h1 .* h1));

