function [e, ehuff] = ent4plot(y,qstep,titlestr,ax,run)

% function [e, ehuff] = ent4plot(y,qstep,titlestr,ax,run)
% Plot the entropy bar graphs for the 4 quadrants of image y, quantised
% to steps of qstep.
% If qstep is a vector, each subimage is quantised separately such that
% qstep(1:4) are applied to subimages [a b; c d] respectiveley.
% e is a column vector of the 4 entropies [ea; eb; ec; ed]
% of the subimages [a b; c d].
% ehuff is like e but is the vector of huffman code entropies.
% titlestr is the title of the whole group of plots.
% ax, if given, defines the axes.
% run, if non-zero, calculates entropies based on run-lengths for the
% zero entries in all but the top left subimage.

if nargin < 5, run = 0; end
if length(qstep) < 4, qstep = qstep(1) * ones(1,4); end

labelfont = 16;
sy = round(size(y)/2);
t1 = 1:sy(1);
t2 = 1:sy(2);
e = zeros(4,1);
ehuff = zeros(4,1);
eng = zeros(4,1);
% return

setfig(gcf)
clf

subplot(2,2,1)
[na,xa] = hist2d(round(y(t1,t2)/qstep(1)));
e(1) = sum(entrop(na,xa));
[bits, huffval, huffent] = genhuff2(na); % huffent = huffman code entropy.
ehuff(1) = sum(huffent);
eng(1) = sum(sum(y(t1,t2).^2));
x0(1) = mean(mean(abs(y(t1,t2))));
text(0,0.9,'  Lo-Lo','units','norm');
text(0,0.8,'  band','units','norm');
xlabel(sprintf('Energy=%.2fe6, entropy=%.2f bits.',eng(1)/1e6,e(1)),'FontSize',labelfont);
if nargin > 3, axis(ax); end
drawnow

if nargin > 2, 
  text(1.2,1.07,titlestr,'Units','norm','horiz','center','vert','bottom')
  drawnow
end

subplot(2,2,2)
if run == 0,
  [nb,xb] = hist2d(round(y(t1,t2+sy(2))/qstep(2)));
  e(2) = sum(entrop(nb,xb));
  [bits, huffval, huffent] = genhuff2(nb); % huffent = huffman code entropy.
else
  [ent, huffent] = runent(round(y(t1,t2+sy(2))/qstep(2)));
  e(2) = sum(ent);
end
ehuff(2) = sum(huffent);
eng(2) = sum(sum(y(t1,t2+sy(2)).^2));
x0(2) = mean(mean(abs(y(t1,t2+sy(2)))));
text(0,0.9,'  Hi-Lo','units','norm');
text(0,0.8,'  band','units','norm');
xlabel(sprintf('Energy=%.2fe6, entropy=%.2f bits.',eng(2)/1e6,e(2)),'FontSize',labelfont);
if nargin > 3, axis(ax); end
drawnow

subplot(2,2,3)
if run == 0,
  [nc,xc] = hist2d(round(y(t1+sy(1),t2)/qstep(3)));
  e(3) = sum(entrop(nc,xc));
  [bits, huffval, huffent] = genhuff2(nc); % huffent = huffman code entropy.
else
  [ent, huffent] = runent(round(y(t1+sy(1),t2)/qstep(3)));
  e(3) = sum(ent);
end
ehuff(3) = sum(huffent);
eng(3) = sum(sum(y(t1+sy(1),t2).^2));
x0(3) = mean(mean(abs(y(t1+sy(1),t2))));
text(0,0.9,'  Lo-Hi','units','norm');
text(0,0.8,'  band','units','norm');
xlabel(sprintf('Energy=%.2fe6, entropy=%.2f bits.',eng(3)/1e6,e(3)),'FontSize',labelfont);
if nargin > 3, axis(ax); end
drawnow

subplot(2,2,4)
if run == 0,
  [nd,xd] = hist2d(round(y(t1+sy(1),t2+sy(2))/qstep(4)));
  e(4) = sum(entrop(nd,xd));
  [bits, huffval, huffent] = genhuff2(nd); % huffent = huffman code entropy.
else
  [ent, huffent] = runent(round(y(t1+sy(1),t2+sy(2))/qstep(4)));
  e(4) = sum(ent);
end
ehuff(4) = sum(huffent);
eng(4) = sum(sum(y(t1+sy(1),t2+sy(2)).^2));
x0(4) = mean(mean(abs(y(t1+sy(1),t2+sy(2)))));
text(0,0.9,'  Hi-Hi','units','norm');
text(0,0.8,'  band','units','norm');
xlabel(sprintf('Energy=%.2fe6, entropy=%.2f bits.',eng(4)/1e6,e(4)),'FontSize',labelfont);
if nargin > 3, axis(ax); end
drawnow

hold off
disp(titlestr),energy=eng',entropy=e',huffent=ehuff',x0

return;


